// SI008Doc.cpp : implementation of the CSI008Doc class
//

#include "stdafx.h"
#include "SI008.h"

#include "SI008Doc.h"
#include "SI008doc.h"
#include "SI008view.h"
#include ".\SI008doc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CSI008Doc

IMPLEMENT_DYNCREATE(CSI008Doc, CDocument)

BEGIN_MESSAGE_MAP(CSI008Doc, CDocument)
	ON_COMMAND(ID_ACTION_CONFIG, OnActionConfig)
	ON_COMMAND(ID_ACTION_SNAP, OnActionSnap)
	ON_COMMAND(ID_ACTION_LIVESTART, OnActionLivestart)
	ON_COMMAND(ID_ACTION_LIVESTOP, OnActionLivestop)
	ON_COMMAND(ID_BLACKREFERENCE_GRAB, OnBlackreferenceGrab)
	ON_COMMAND(ID_BLACKREFERENCE_SAVE, OnBlackreferenceSave)
	ON_COMMAND(ID_BLACKREFERENCE_LOAD, OnBlackreferenceLoad)
	ON_COMMAND(ID_FILE_SAVE, OnFileSave)	
	ON_COMMAND(ID_ACTION_CHANGEGAIN32782, OnActionChangeGainB)
END_MESSAGE_MAP()


// CSI008Doc construction/destruction

CSI008Doc::CSI008Doc()
{	
	m_strConfigFile = _T("");
	m_CaptureInitialized = FALSE;
	m_iCamID = 0;
	m_dwFrame = 0;
	m_Status = Stop;
}

CSI008Doc::~CSI008Doc()
{
}


DWORD WINAPI CSI008Doc::LiveThreadFunc(LPVOID pParam)
{
	CSI008Doc * pDoc = static_cast<CSI008Doc*>(pParam);

	if(!pDoc->InitCapture()) return 1;
	

	pDoc->m_bStopLiveFlag = FALSE;
	pDoc->m_dwFrame = 0;

	BITMAPINFO * pPackedDIB=NULL;
	SIResult sr;
	while(!pDoc->m_bStopLiveFlag)
	{
		::Sleep(1);

		sr = SI_CaptureDIB(&pDoc->m_CapCxt);

		if(sr != SR_OK)
		{
			printf("bad return code %d\n",sr);
			continue;
		}
		else
			printf("good return code %d\n",sr);

		pDoc->m_dwFrame++;

		pDoc->ShowImage(&pDoc->m_CapCxt.PackedDIB);

		if(pDoc->m_bStopLiveFlag)
			break;		
	}

	pDoc->EndCapture();

	return 0;
}


BOOL CSI008Doc::InitCapture()
{
	if(m_CaptureInitialized) return TRUE;
	SICameraModel mod;
	if(SI_DetectCameraModel(0,&mod) != SR_OK)
	{
		AfxMessageBox("Camera model detection failure, assuming generic.");
		mod = SIGenericCam;
	}

	// Load the camera configuration
	if(m_strConfigFile == _T(""))
	{
		AfxMessageBox("Please select the configuration files first (Action | Config).");
		return FALSE;
	}
	SI_ReadConfig(m_strConfigFile,&m_Config, mod);	

	if(SI_BeginCapture(&m_Config, &m_CapCxt) != SR_OK)
 	{
		AfxMessageBox("Initializing capture failed",MB_OK|MB_ICONSTOP);
		return FALSE;
	}

	m_CaptureInitialized = TRUE;
	return TRUE;	
}

void CSI008Doc::EndCapture()
{
	if(!m_CaptureInitialized) 
		return;
	
	SI_EndCapture(&m_CapCxt);
	m_CaptureInitialized = FALSE;
}


void CSI008Doc::ShowImage(const SIPackedDIB * pPackedDIB)
{
	POSITION pos = GetFirstViewPosition();
	CSI008View * pView = static_cast<CSI008View*>(GetNextView( pos ));
	pView->ShowImage(pPackedDIB);	
}

// CSI008Doc commands

void CSI008Doc::OnActionConfig()
{
	TCHAR szFilters[] = _T("Config files (*.cfg)|*.cfg|All files(*.*)|*.*||");
	CFileDialog  cfdlg(TRUE, _T("cfg"), _T("*.cfg"), OFN_FILEMUSTEXIST|OFN_HIDEREADONLY|OFN_EXPLORER, szFilters, NULL);	
	if (IDOK == cfdlg.DoModal())
		m_strConfigFile = cfdlg.GetPathName();
}

void CSI008Doc::OnActionSnap()
{
	if(!InitCapture()) 
		return;
	
	SI_CaptureDIB(&m_CapCxt);

	ShowImage(&m_CapCxt.PackedDIB);
}

void CSI008Doc::OnActionLivestart()
{
	if(m_Status != Run)
	{
		m_hLiveThread = ::CreateThread(NULL,0,LiveThreadFunc,this,CREATE_SUSPENDED,NULL);
		m_Status = Run;
		::ResumeThread(m_hLiveThread);
	}	
}

void CSI008Doc::OnActionLivestop()
{
	if(m_Status == Run)
	{
		m_bStopLiveFlag = TRUE;
		::WaitForSingleObject(m_hLiveThread,INFINITE);
		::CloseHandle(m_hLiveThread);
		m_Status = Stop;
	}	

}

void CSI008Doc::OnBlackreferenceGrab()
{
	if(!InitCapture()) return;

	AfxMessageBox("Please cover your camera lens");

	// Stop the live thread before grab the black reference image.
	if(m_Status == Run)
		::SuspendThread(m_hLiveThread);

	m_CapCxt.iApplyBlackRefImage = 1;
	SI_CaptureBlackReferenceImage(&m_CapCxt);	

	// Resume the live thread.
	if(m_Status == Run)
		::ResumeThread(m_hLiveThread);

}

void CSI008Doc::OnBlackreferenceSave()
{
	if(!InitCapture()) return;

	// Retrieve black reference image previously captured.
	if( SI_GetCapturedBlackReferenceImage(&m_CapCxt) != SR_OK)
		return;

	TCHAR szFilters[] = _T("Bin files (*.bin)|*.bin|All files(*.*)|*.*||");
	CFileDialog  cfdlg(TRUE, _T("bin"), _T("*.bin"), OFN_OVERWRITEPROMPT|OFN_HIDEREADONLY|OFN_EXPLORER, szFilters, NULL);	

	if (IDOK == cfdlg.DoModal())
	{
		CString strBinFile = cfdlg.GetPathName();

		// Save the black reference image as a bin file.
		SI_SaveBinFile(&m_CapCxt.BlackRefImage,(LPCTSTR)strBinFile);
	}	
}

void CSI008Doc::OnBlackreferenceLoad()
{
	if(!InitCapture()) return;

	TCHAR szFilters[] = _T("Config files (*.bin)|*.bin|All files(*.*)|*.*||");
	CFileDialog  cfdlg(TRUE, _T("bin"), _T("*.bin"), OFN_FILEMUSTEXIST|OFN_HIDEREADONLY|OFN_EXPLORER, szFilters, NULL);	
	if (IDOK == cfdlg.DoModal())
	{
		CString strBRBinFile = cfdlg.GetPathName();

		// Load black reference image directly from a bin file.
		SI_LoadBlackReferenceImageFromFile(m_iCamID,(const char*)strBRBinFile);
	}

}

void CSI008Doc::OnFileSave()
{	
	if(!m_CaptureInitialized)
	{
		AfxMessageBox("Capture has not started.");
		return;
	}

	TCHAR szFilters[] = _T("Bin files (*.bin)|*.bin|All files(*.*)|*.*||");
	CFileDialog  cfdlg(TRUE, _T("bin"), _T("*.bin"), OFN_OVERWRITEPROMPT|OFN_HIDEREADONLY|OFN_EXPLORER, szFilters, NULL);	

	if (IDOK == cfdlg.DoModal())
	{
		CString strBinFile = cfdlg.GetPathName();

		SI_GetLastCapturedBinImage(&m_CapCxt);
		SI_SaveBinFile(&m_CapCxt.BayerImage,(LPCTSTR)strBinFile);
	}	
}

void CSI008Doc::OnCloseDocument()
{	
	EndCapture();
	CDocument::OnCloseDocument();	
}



void CSI008Doc::OnActionChangeGainB()
{
	OnActionLivestop();
	EndCapture();

	// increment gain index by 5
	m_Config.CameraConfig.iGlobalGain += 5;
	Sleep(10);
	if(SI_BeginCapture(&m_Config, &m_CapCxt) != SR_OK)
 	{
		AfxMessageBox("Initializing capture failed",MB_OK|MB_ICONSTOP);
		return;
	}

	m_CaptureInitialized = TRUE;
	OnActionLivestart();
}
	
