#ifndef __SIIPL_H__
#define __SIIPL_H__

#pragma once 


#include <sidefs.h>


#undef SIAPI
#ifdef SIIPL_EXPORTS
#define SIAPI __declspec(dllexport)
#else
#define SIAPI __declspec(dllimport)
#endif

typedef struct tagSIIplConfig
{
	int8		iBlackReferenceCorrection;
	int8		iAutoDetectDeviantPixel;
	int8		iCorrectDeviantPixel;
}SIIplConfig;

#ifdef __cplusplus
extern "C" {
#endif

SIAPI SIResult			SI_AllocMonoImage(SIMonoImage * pImg);
SIAPI	void					SI_FreeMonoImage(SIMonoImage * pImg);
SIAPI SIResult			SI_AllocColorImage(SIColorImage * pImg);
SIAPI void					SI_FreeColorImage(SIColorImage * pImg);
SIAPI BITMAPINFO *	SI_AllocDIB(int iWidth, int iHeight, int iBpp);
SIAPI void					SI_FreeDIB(BITMAPINFO * pBmp);
SIAPI SIResult			SI_CopyMonoImage(const SIMonoImage * pSrc, SIMonoImage * pDst);

SIAPI SIResult	SI_YCrCb422ToRGB(SIMonoImage *pSrc, SIColorImage * pDst);
SIAPI SIResult	SI_BayerToRGBP3(SIMonoImage * pSrc, SIColorImage * pDst, SIBayerPhase bp, SIBayerInterpAlgorithm algo);
SIAPI SIResult	SI_ConvertRGBP3ToC3(const SIColorImage * pSrc, SIColorImage * pDst);
SIAPI SIResult	SI_ConvertRGBToDIB(SIColorImage * pClrImg, BITMAPINFO * pBmp);

SIAPI void			SI_BuildColorMatrix(SI32f SatFactor, SI32f ClrMat[3][3]);
SIAPI void			SI_CalculateAutoWhiteBalance(const SIColorImage * pSrc, float fWBCoef[3], SIWhiteBalanceAlgorithm wb);
SIAPI SIResult	SI_ApplyWhiteBalanceGains(SIColorImage * pImg, float fGains[3]);
SIAPI SIResult	SI_ColorProcessing(SIColorImage * pSrc, SIColorImage * pDst, const float fSatMat[3][3], const float fRGBGain[3],
                                   float fGlobalGain, int iGlobalOffset, float fWBCoef[3],SIWhiteBalanceAlgorithm WBAlgo);
SIAPI void			SI_OffsetMonoImage(const SIMonoImage * pSrc, SIMonoImage * pDst, int iOffset);
SIAPI SIResult	SI_SubtractBlackReference(SIMonoImage * pImg, SIMonoImage * pBlackRef);
SIAPI SIResult	SI_SwapBR(SIColorImage * pSrc, SIColorImage * pDst);
SIAPI SIResult	SI_DeviantPixelDetection(SIMonoImage * pBayer, SIDPCAlgorithm DPCAlgo,int *iDPNum, SIPoint * pDPList, int iListLength);
SIAPI SIResult	SI_DeviantPixelCorrection(SIMonoImage * pBayer, int iDPNum, SIPoint * pDPList);
SIAPI SIResult	SI_BitReduction_Mono(SIMonoImage * pSrc, SIMonoImage * pDst, SIBitReductionAlgorithm bralgo);
SIAPI SIResult	SI_BitReduction_Color(SIColorImage * pSrc, SIColorImage * pDst, SIBitReductionAlgorithm bralgo);

SIAPI SIResult	SI_LoadBinFile(const char * lpszFileName,	SIMonoImage * pImg);
SIAPI SIResult  SI_SaveBinFile(const SIMonoImage * pImg, const char * lpszFileName);
//SIAPI SIResult	SI_SaveDIBFile(const BITMAPINFO * pPackedDIB, const char * lpszFileName);
SIAPI SIResult	SI_SavePackedDIBAsBmp(const SIPackedDIB * pPackedDIB, const char * lpszFileName);
SIAPI SIResult	SI_SaveAsBmp_Color(const SIColorImage * pImg, const char * lpszFileName);
SIAPI SIResult	SI_SaveAsBmp_Mono(const SIMonoImage * pImg, const char * lpszFileName);

SIAPI SIResult	SI_InitVFE(SIU_VFEConfig * pCfg, SIU_VFEContext * pCxt);
SIAPI SIResult	SI_ExitVFE(SIU_VFEConfig * pCfg, SIU_VFEContext * pCxt);
SIAPI SIResult	SI_RunVFESync(uint8 * pBayer, SIU_VFEConfig * pCfg, SIU_VFEContext * pCxt);

#ifdef __cplusplus
};
#endif

#endif